/*
**
**  WAY (Where Are You ?)
**
**  dbase.h		definitions for the database module.
**
**  Author: Roberto Bagnara (bagnara@vxcern.cern.ch),
**          CERN  DD Division OC Group RA Section.
**
**  All rights reserved to:
**
**          CERN
**          European Laboratory for Particle Physics
**          CH 1211 Geneva 23
**          SWITZERLAND
**
**  Permission is granted to any individual or institution
**  to use, copy, or redistribute this software so long as
**  it  is not  sold for  profit, provided  this copyright
**  notice is retained.
**
**  Modification History:
**
**  Version  Date    Who              Comments
**
**  1.0.00   890728  Roberto Bagnara  First working version
**
**
**  Description:
**
**  this module contains the data structure definitions for the database
**  module.
**
*/


/*
**      Forward references to structures.
**      ---------------------------------
*/

typedef struct list_node LIST_NODE, *LIST_PTR;
typedef struct tree_node TREE_NODE, *TREE_PTR;

/*
**      Now the real structures.
**      ------------------------
**
**  Services and addresses are stored into two trees. Every node of such
**  trees point to a linked list of association to nodes of the other tree.
**  Thus if node S is associated to node A, an association item will be
**  on both the association list of node S and the association list of
**  node A.
*/

struct tree_node {
    char     *name;		/* Service or address */
    TREE_PTR  left;		/* Left child */
    TREE_PTR  right;		/* Right child */
    LIST_PTR  assoc;		/* Head of association list */
};

struct list_node {
    TREE_PTR s_node;		/* One end of the link */
    TREE_PTR a_node;		/* The other end of the link */
    LIST_PTR s_next;		/* Next node in S to A association list */
    LIST_PTR a_next;		/* Next node in A to S association list */
    INFO     info;		/* Additional informations */
};

/*
**      The null pointers.
**      ------------------
*/

#define NULL_TREE_PTR (TREE_PTR) 0
#define NULL_LIST_PTR (LIST_PTR) 0
