/*
**
**  WAY (Where Are You ?)
**
**  way2way.c		WAY to WAY communication module.
**
**  Author: Roberto Bagnara (bagnara@vxcern.cern.ch),
**          CERN  DD Division OC Group RA Section.
**
**  All rights reserved to:
**
**          CERN
**          European Laboratory for Particle Physics
**          CH 1211 Geneva 23
**          SWITZERLAND
**
**  Permission is granted to any individual or institution
**  to use, copy, or redistribute this software so long as
**  it  is not  sold for  profit, provided  this copyright
**  notice is retained.
**
**  Modification History:
**
**  Version  Date    Who              Comments
**
**  ?.?.??   891116  Roberto Bagnara  Devised
**
**
**  Description:
**
**  this module contains the procedures a WAY server uses to communicate
**  with other WAY servers. Its utility is not clear now, but could be such
**  in the future.
**
*/

/*
**	Include files.
**	--------------
*/

#include "way.h"
#include "info.h"
#include "dbase.h"

/*
**      External functions.
**      -------------------
*/

#ifdef __STDC__
extern void db_associate(const char *s_name, const char *a_name, const INFO info);
extern void db_visit_a(int (*action)(TREE_PTR node));
#else
extern void db_associate();
extern void db_visit_a();
#endif

/*
**	ALL THIS DOES NOT EXIST YET !!!
**	-------------------------------
*/

typedef    struct {
      long flags;
      long long_info[4];
   } info_type;

#ifdef __STDC__
static int do_download(TREE_PTR node)
#else
static int do_download(node)
TREE_PTR node;
#endif
{
    LIST_PTR tmp;
    long status;
    info_type info;
    int i;

    for (tmp=node->assoc; tmp; tmp = tmp->a_next) {

    	info.flags = tmp->info.flags;
    	for (i = 0; i < 4; i++)
    	    info.long_info[i] = tmp->info.long_info.array[i];

	status = way_2w_store_rem(tmp->s_node->name, node->name, info);
	if (status != WAY_S_NORMAL)
	    return 1;
    }
    return 0;
}

way_status way_2w_download_loc()
{
    db_visit_a(do_download);
    return WAY_S_NORMAL;
}

way_status way_2w_store_loc(service, address, info)
char *service;
char *address;
info_type info;
{
    INFO db_info;
    int i;

    db_info.flags = info.flags;
    for (i = 0; i < 4; i++)
	db_info.long_info.array[i] = info.long_info[i];
    db_associate(service, address, db_info);
    return WAY_S_NORMAL;
}
